import { readBody, defineEventHandler, createError } from "h3";
import { getMultiCacheContext } from "./../../helpers/server.mjs";
import { DEFAULT_CACHE_TAG_INVALIDATION_DELAY } from "./../../settings.mjs";
import { checkAuth } from "./helpers/index.mjs";
import { useRuntimeConfig } from "#imports";
import {
  decodeComponentCacheItem,
  decodeRouteCacheItem
} from "../../helpers/cacheItem.mjs";
async function getTagsToPurge(event) {
  const body = await readBody(event);
  if (body && Array.isArray(body)) {
    return body;
  }
  throw createError({
    statusCode: 400,
    statusMessage: "No valid tags provided."
  });
}
function onlyUnique(value, index, self) {
  return self.indexOf(value) === index;
}
export class DebouncedInvalidator {
  /**
   * Buffer of tags to invalidate in the next run.
   */
  tags;
  /**
   * Debounce delay.
   */
  delay;
  /**
   * The current timeout ID for the next run. Reset after each run.
   */
  timeout;
  /**
   * The cache context instance.
   */
  cacheContext;
  constructor() {
    this.tags = [];
    this.timeout = null;
    this.delay = DEFAULT_CACHE_TAG_INVALIDATION_DELAY;
  }
  setDelay(delay = DEFAULT_CACHE_TAG_INVALIDATION_DELAY) {
    this.delay = delay;
  }
  /**
   * Add tags to be purged.
   */
  add(tags = []) {
    tags.forEach((tag) => {
      if (!this.tags.includes(tag)) {
        this.tags.push(tag);
      }
    });
    if (!this.timeout) {
      this.timeout = setTimeout(() => {
        this.invalidate();
      }, this.delay);
    }
  }
  async getCacheTags(cacheName, key) {
    if (cacheName === "data") {
      const item = await this.cacheContext?.[cacheName]?.getItem(key);
      if (item && typeof item === "object" && item !== null) {
        return item.cacheTags;
      }
    } else if (cacheName === "route") {
      const cached = await this.cacheContext?.[cacheName]?.getItemRaw(key);
      if (cached) {
        return decodeRouteCacheItem(cached)?.cacheTags;
      }
    } else if (cacheName === "component") {
      const cached = await this.cacheContext?.[cacheName]?.getItemRaw(key);
      if (cached) {
        return decodeComponentCacheItem(cached)?.cacheTags;
      }
    }
  }
  /**
   * Invalidate the tags in the buffer.
   */
  async invalidate() {
    if (!this.cacheContext) {
      return;
    }
    const tags = this.tags.filter(onlyUnique);
    this.tags = [];
    this.timeout = null;
    let key;
    for (key in this.cacheContext) {
      const cache = this.cacheContext[key];
      if (cache) {
        const cacheItemKeys = await cache.getKeys();
        for (const cacheKey of cacheItemKeys) {
          const itemCacheTags = await this.getCacheTags(key, cacheKey);
          if (itemCacheTags) {
            const shouldPurge = itemCacheTags.some((v) => {
              return tags.includes(v);
            });
            if (shouldPurge) {
              await cache.removeItem(cacheKey);
            }
          }
        }
      }
    }
    return true;
  }
}
const invalidator = new DebouncedInvalidator();
export default defineEventHandler(async (event) => {
  await checkAuth(event);
  const tags = await getTagsToPurge(event);
  if (!invalidator.cacheContext) {
    invalidator.cacheContext = getMultiCacheContext(event);
    const { multiCache } = useRuntimeConfig();
    invalidator.setDelay(multiCache?.api?.cacheTagInvalidationDelay);
  }
  invalidator.add(tags);
  return {
    status: "OK",
    tags
  };
});
